package dbtypes

import (
	"time"

	"git.andr3h3nriqu3s.com/andr3/fyp/logic/db"
)

type DefinitionStatus int

const (
	DEFINITION_STATUS_CANCELD_TRAINING DefinitionStatus = -4
	DEFINITION_STATUS_FAILED_TRAINING                   = -3
	DEFINITION_STATUS_PRE_INIT                          = 1
	DEFINITION_STATUS_INIT                              = 2
	DEFINITION_STATUS_TRAINING                          = 3
	DEFINITION_STATUS_PAUSED_TRAINING                   = 6
	DEFINITION_STATUS_TRANIED                           = 4
	DEFINITION_STATUS_READY                             = 5
)

type Definition struct {
	Id             string    `db:"md.id" json:"id"`
	ModelId        string    `db:"md.model_id" json:"model_id"`
	Accuracy       float64   `db:"md.accuracy" json:"accuracy"`
	TargetAccuracy int       `db:"md.target_accuracy" json:"target_accuracy"`
	Epoch          int       `db:"md.epoch" json:"epoch"`
	Status         int       `db:"md.status" json:"status"`
	CreatedOn      time.Time `db:"md.created_on" json:"created"`
	EpochProgress  int       `db:"md.epoch_progress" json:"epoch_progress"`
}

type SortByAccuracyDefinitions []*Definition

func (nf SortByAccuracyDefinitions) Len() int      { return len(nf) }
func (nf SortByAccuracyDefinitions) Swap(i, j int) { nf[i], nf[j] = nf[j], nf[i] }
func (nf SortByAccuracyDefinitions) Less(i, j int) bool {
	return nf[i].Accuracy < nf[j].Accuracy
}

func GetDefinition(db db.Db, definition_id string) (definition Definition, err error) {
	err = GetDBOnce(db, &definition, "model_definition as md where id=$1;", definition_id)
	return
}

func MakeDefenition(db db.Db, model_id string, target_accuracy int) (definition Definition, err error) {
	var NewDefinition = struct {
		ModelId        string `db:"model_id"`
		TargetAccuracy int    `db:"target_accuracy"`
	}{ModelId: model_id, TargetAccuracy: target_accuracy}

	id, err := InsertReturnId(db, &NewDefinition, "model_definition", "id")
	if err != nil {
		return
	}
	return GetDefinition(db, id)
}

func (d Definition) UpdateStatus(db db.Db, status DefinitionStatus) (err error) {
	_, err = db.Exec("update model_definition set status=$1 where id=$2", status, d.Id)
	return
}

func (d Definition) MakeLayer(db db.Db, layer_order int, layer_type LayerType, shape string) (layer Layer, err error) {
	var NewLayer = struct {
		DefinitionId string    `db:"def_id"`
		LayerOrder   int       `db:"layer_order"`
		LayerType    LayerType `db:"layer_type"`
		Shape        string    `db:"shape"`
	}{
		DefinitionId: d.Id,
		LayerOrder:   layer_order,
		LayerType:    layer_type,
		Shape:        shape,
	}

	id, err := InsertReturnId(db, &NewLayer, "model_definition_layer", "id")
	if err != nil {
		return
	}

	return GetLayer(db, id)
}

func (d Definition) GetLayers(db db.Db, filter string, args ...any) (layer []*Layer, err error) {
	args = append(args, d.Id)
	return GetDbMultitple[Layer](db, "model_definition_layer as mdl where mdl.def_id=$1 "+filter, args...)
}

func (d *Definition) UpdateAfterEpoch(db db.Db, accuracy float64, epoch int) (err error) {
	d.Accuracy = accuracy
	d.Epoch += epoch
	_, err = db.Exec("update model_definition set epoch=$1, accuracy=$2 where id=$3", d.Epoch, d.Accuracy, d.Id)
	return
}
