package dbtypes

import (
	"encoding/json"
	"fmt"

	"git.andr3h3nriqu3s.com/andr3/fyp/logic/db"
	"github.com/charmbracelet/log"
)

type LayerType int

const (
	LAYER_INPUT        LayerType = 1
	LAYER_DENSE                  = 2
	LAYER_FLATTEN                = 3
	LAYER_SIMPLE_BLOCK           = 4
)

type Layer struct {
	Id           string    `db:"mdl.id" json:"id"`
	DefinitionId string    `db:"mdl.def_id" json:"definition_id"`
	LayerOrder   int       `db:"mdl.layer_order" json:"layer_order"`
	LayerType    LayerType `db:"mdl.layer_type" json:"layer_type"`
	Shape        string    `db:"mdl.shape" json:"shape"`
	ExpType      int       `db:"mdl.exp_type" json:"exp_type"`
}

func (x *Layer) ShapeToSize() {
	v := x.GetShape()
	switch x.LayerType {
	case LAYER_INPUT:
		x.Shape = fmt.Sprintf("%d,%d", v[1], v[2])
	case LAYER_DENSE:
		x.Shape = fmt.Sprintf("(%d)", v[0])
	default:
		x.Shape = "ERROR"
	}
}

func ShapeToString(args ...int) string {
	text, err := json.Marshal(args)
	if err != nil {
		log.Error("json err!", "err", err)
		panic("Could not generate Shape")
	}
	return string(text)
}

func StringToShape(str string) (shape []int64) {
	err := json.Unmarshal([]byte(str), &shape)
	if err != nil {
		log.Error("json err!", "err", err)
		panic("Could not parse Shape")
	}
	return
}

func (l Layer) GetShape() []int64 {
	if l.Shape == "" {
		return []int64{}
	}
	return StringToShape(l.Shape)
}

func GetLayer(db db.Db, layer_id string) (layer Layer, err error) {
	err = GetDBOnce(db, &layer, "model_definition_layer as mdl where mdl.id=$1", layer_id)
	return
}
