type NotificationType = 'danger' | 'success' | 'info';
type Notification = {
	type?: NotificationType;
	timeToLive?: number;
	message: string;
	html?: string;
};

export function createNotificationStore(defaultTimetoLive: number) {
	type InternalNotifications = {
		notification: Notification;
		endDate: Date;
	};
	let notifications = $state<InternalNotifications[]>([]);
	let timeout = $state<number | undefined>();

	function clearNotifications() {
		const now = new Date().getTime();
		let min: number | undefined = undefined;
		notifications = notifications.filter((a) => {
			const t = a.endDate.getTime();
			if (t <= now) {
				return false;
			}
			if (min == undefined) {
				min = t;
			} else if (min > t) {
				min = t;
			}
			return true;
		});
		if (min != undefined) {
			timeout = setTimeout(clearNotifications, now - min);
		} else {
			timeout = undefined;
		}
	}

	function add(noti: Notification) {
		if (!timeout) {
			timeout = setTimeout(clearNotifications, noti.timeToLive ?? defaultTimetoLive);
		}
		const now = new Date();
		notifications.push({
			notification: noti,
			endDate: new Date(now.getTime() + (noti.timeToLive ?? defaultTimetoLive))
		});
	}

	return {
		get notifications() {
			return notifications.map((a) => a.notification);
		},
		clear() {
			notifications.forEach((a) => clearInterval(a.timeout));
			notifications = [];
		},
		add,
		notify(message: string, type: NotificationType = 'danger') {
			add({ message, type });
		},
		display(message: string) {
			add({ message, type: 'danger' });
		}
	};
}

export const notificationStore = createNotificationStore(5000);
